@@ -72,7 +72,7 @@
 #include <getopt.h>
 #include <time.h>
 
-extern void gnuzip(int in, int out, char *origname, unsigned long timestamp, int level, int osflag, int rsync);
+extern void gnuzip(int in, int out, char *origname, unsigned long timestamp, int level, int osflag, int rsync, int newrsync);
 extern void old_bzip2(int level);
 
 #define BUFLEN		(64 * 1024)
@@ -128,7 +128,7 @@ static	void	maybe_err(const char *fmt, ...)
     __attribute__((__format__(__printf__, 1, 2),noreturn));
 static	void	maybe_errx(const char *fmt, ...)
     __attribute__((__format__(__printf__, 1, 2),noreturn));
-static	void	gz_compress(int, int, const char *, uint32_t, int, int, int, int);
+static	void	gz_compress(int, int, const char *, uint32_t, int, int, int, int, int);
 static	void	usage(void);
 static	void	display_version(void);
 static	void	display_license(void);
@@ -145,7 +145,6 @@ static const struct option longopts[] = {
 	{ "no-name",		no_argument,		0,	'n' },
 	{ "name",		no_argument,		0,	'N' },
 	{ "quiet",		no_argument,		0,	'q' },
-	{ "recursive",		no_argument,		0,	'r' },
 	{ "fast",		no_argument,		0,	'1' },
 	{ "best",		no_argument,		0,	'9' },
 	{ "ascii",		no_argument,		0,	'a' },
@@ -154,6 +153,7 @@ static const struct option longopts[] = {
 	{ "old-bzip2",          no_argument,            0,      'O' },
 	{ "zlib",               no_argument,            0,      'Z' },
 	{ "rsyncable",          no_argument,            0,      'R' },
+	{ "new-rsyncable",      no_argument,            0,      'r' },
 	{ "no-timestamp",	no_argument,		0,	'm' },
 	{ "force-timestamp",	no_argument,		0,	'M' },
 	{ "timestamp",		required_argument,	0,	'T' },
@@ -173,8 +173,10 @@ main(int argc, char **argv)
 	const char *progname = argv[0];
 	int gnu = 0;
 	int bzold = 0;
+	int quirks = 0;
 	char *origname = NULL;
 	uint32_t timestamp = 0;
+	int memlevel = 8; /* zlib's default */
 	int nflag = 0;
 	int mflag = 0;
 	int fflag = 0;
@@ -183,6 +185,7 @@ main(int argc, char **argv)
 	int level = 6;
 	int osflag = GZIP_OS_UNIX;
 	int rsync = 0;
+	int new_rsync = 0;
 	int ch;
 
 	if (strcmp(progname, "gunzip") == 0 ||
@@ -238,6 +241,7 @@ main(int argc, char **argv)
 			origname = optarg;
 			break;
 		case 'k':
+			quirks = 1;
 			if (strcmp(optarg, "buggy-bsd") == 0) {
 				/* certain archives made with older versions of
 				 * BSD variants of gzip */
@@ -255,6 +259,16 @@ main(int argc, char **argv)
 				mflag = 1;
 				/* osflag is NTFS */
 				osflag = GZIP_OS_NTFS;
+			} else if (strcmp(optarg, "perl") == 0) {
+				/* Perl's Compress::Raw::Zlib */
+				memlevel = 9;
+
+				/* no name or timestamp information */
+				nflag = 1;
+				mflag = 1;
+				/* maximum compression but without indicating so */
+				level = 9;
+				xflag = 0;
 			} else {
 				fprintf(stderr, "%s: unknown quirk!\n", progname);
 				usage();
@@ -267,8 +281,7 @@ main(int argc, char **argv)
 			rsync = 1;
 			break;
 		case 'r':
-			fprintf(stderr, "%s: recursive is not supported on this version\n", progname);
-			usage();
+			new_rsync = 1;
 			break;
 		case 'd':
 			fprintf(stderr, "%s: decompression is not supported on this version\n", progname);
@@ -305,24 +318,24 @@ main(int argc, char **argv)
 		timestamp = 0;
 
 	if (gnu) {
-		if (ntfs_quirk || xflag >= 0) {
+		if (quirks) {
 			fprintf(stderr, "%s: quirks not supported with --gnu\n", progname);
 			return 1;
 		}
-		gnuzip(STDIN_FILENO, STDOUT_FILENO, origname, timestamp, level, osflag, rsync);
+		gnuzip(STDIN_FILENO, STDOUT_FILENO, origname, timestamp, level, osflag, rsync, new_rsync);
 	} else if (bzold) {
-		if (ntfs_quirk || xflag >= 0) {
+		if (quirks) {
 			fprintf(stderr, "%s: quirks not supported with --old-bzip\n", progname);
 			return 1;
 		}
 		old_bzip2(level);
 	} else {
-		if (rsync) {
+		if (rsync || new_rsync) {
 			fprintf(stderr, "%s: --rsyncable not supported with --zlib\n", progname);
 			return 1;
 		}
 
-		gz_compress(STDIN_FILENO, STDOUT_FILENO, origname, timestamp, level, osflag, xflag, ntfs_quirk);
+		gz_compress(STDIN_FILENO, STDOUT_FILENO, origname, timestamp, level, memlevel, osflag, xflag, ntfs_quirk);
 	}
 	return 0;
 }
@@ -357,7 +370,7 @@ maybe_errx(const char *fmt, ...)
 
 /* compress input to output. */
 static void
-gz_compress(int in, int out, const char *origname, uint32_t mtime, int level, int osflag, int xflag, int ntfs_quirk)
+gz_compress(int in, int out, const char *origname, uint32_t mtime, int level, int memlevel, int osflag, int xflag, int ntfs_quirk)
 {
 	z_stream z;
 	char *outbufp, *inbufp;
@@ -396,7 +409,7 @@ gz_compress(int in, int out, const char *origname, uint32_t mtime, int level, in
 	z.avail_out = BUFLEN - i;
 
 	error = deflateInit2(&z, level, Z_DEFLATED,
-			     (-MAX_WBITS), 8, Z_DEFAULT_STRATEGY);
+			     (-MAX_WBITS), memlevel, Z_DEFAULT_STRATEGY);
 	if (error != Z_OK)
 		maybe_err("deflateInit2 failed");
 
@@ -515,8 +528,9 @@ usage(void)
     " -T --timestamp SECONDS   set the timestamp to the specified number of seconds\n"
     " \ngnu-specific options:\n"
     " -R --rsyncable           make rsync-friendly archive\n"
+    " -r --new-rsyncable       make rsync-friendly archive (new version)\n"
     " \nzlib-specific options:\n"
-    " -k --quirk QUIRK         enable a format quirk (buggy-bsd, ntfs)\n");
+    " -k --quirk QUIRK         enable a format quirk (buggy-bsd, ntfs, perl)\n");
 	exit(0);
 }
 
